/* This is the main class for J-Offline, an dialup internet connection
   watchdog, which dials a backup internet connection in case the main
   link fails.

   The code is not object-orientated - it is merely a simple script written
   in Java.

   This class controls the main execution flow of the program. For more info,
   including diagrams,see <http://www.bucksch.org/1/projects/various/joffline>.

   (c) 2003 Ben Bucksch <http://www.bucksch.org>.
   New-BSD-style license.
*/

package org.bucksch.tools.joffline;

class JOffline
{
  static String ml = "ppp0"; // main link
  static String bl = "ippp0"; // backup link
  // you need to edit the start/close scripts and servers in SysCall as well

  public JOffline()
  {}

  public void run()
  {
    while (true) // loop endlessly to keep connections up until we're killed
    {
      if (!SysCall.internetWorks(""))
      {
        SysCall.log("inet connection broken. waiting 5 mins, " +
                    "if it comes back online by itself (redial, " +
                    "temp. failures etc.).");
        SysCall.sleep(300);
        if (!SysCall.internetWorks(""))
        {
          SysCall.log("inet connection still broken, dialing backup.");
          SysCall.ifaceDialClose(bl, true); // dial ISDN backup
          SysCall.sleep(25); // wait for it to come up
          SysCall.setDefaultRoute(bl); // important to advertize ip address
          if (!(SysCall.ifaceIsUp(bl) && SysCall.internetWorks("")))
          {
            SysCall.log("seems like backup doesn't work either. " +
                               "giving up.");
            // Dead!
          }
          else
          {
            SysCall.log("backup internet connection up and working.");
            // $$!

            // We won't check, if the backup line goes down as well,
            // because we can't do anything anyways. (assumption: only 2 conn.)
          }

          // While the backup connection is up, try to bring the main
          // connection back online. when that succeeds, switch back to that.
          SysCall.log("trying to bring main internet connection " +
                             "back up.");
          while (!(SysCall.ifaceIsUp(ml) && SysCall.internetWorks(ml)))
          {
            // check,if the interface is up, but broken, i.e. if only the first
            // of the above conditions is true. in that case, redial.
            if (SysCall.ifaceIsUp(ml))
            {
              SysCall.ifaceDialClose(ml, false); // close broken link
            }

            SysCall.ifaceDialClose(ml, true); // try to bring up main link
            SysCall.sleep(300); // give it 5 minutes to come up (autoredials)
          }

          // ahh!!!
          SysCall.log("it seems that the main link came back online");
          SysCall.setDefaultRoute(ml);
          if (SysCall.internetWorks(""))
          {
            SysCall.log("and it seems to work, so closing backup link");
            SysCall.ifaceDialClose(bl, false);
            if (!SysCall.internetWorks(""))
            {
              // maybe bl's close script messed with the default route
              SysCall.setDefaultRoute(ml);
              if (!SysCall.internetWorks(""))
              {
                // shit
                SysCall.log("What the heck? It doesn't work anymore!");
              }
            }
          }
          else
          {
            SysCall.log("What the heck? It doesn't work. :-(");
          }
        }
        else
        {
          SysCall.log("inet connection works again");
        }
      }
      SysCall.log("ok");
      SysCall.sleep(300); //check inet connection that often under normal cond.
    }
  }

  public static void main(String[] args)
  {
    JOffline app = new JOffline();
    app.run();
  }
}
